local dialog = nil
local vb = nil

class "RenoiseScriptingTool" (renoise.Document.DocumentNode)
function RenoiseScriptingTool:__init()    
  renoise.Document.DocumentNode.__init(self) 
  self:add_property("Name", "Untitled Tool")
  self:add_property("Version", "1.0")
end

local manifest = RenoiseScriptingTool()
local ok,err = manifest:load_from("manifest.xml")
local tool_name = manifest:property("Name").value
local tool_version = manifest:property("Version").value

local function round(value)
  return math.floor(value + 0.5)
end

local function set_bpm_from_sample()
  local rs = renoise.song()
  if (rs.selected_sample.sample_buffer.has_sample_data) then
    local lines_per_beat = rs.transport.lpb
    local sample_frames = rs.selected_sample.sample_buffer.number_of_frames
    local sample_rate = rs.selected_sample.sample_buffer.sample_rate
    local sample_seconds = sample_frames / sample_rate
    local pattern_lines = rs.selected_pattern.number_of_lines
    local seconds_per_line = pattern_lines / sample_seconds
    local beats_per_minute = (seconds_per_line / lines_per_beat) * 60
    rs.transport.bpm = math.min(999, math.max(32, beats_per_minute))
  end
end

local function set_pattern_length_from_sample(set_bpm)
  local rs = renoise.song()
  if (rs.selected_sample.sample_buffer.has_sample_data) then
    local beats_per_minute = rs.transport.bpm
    local lines_per_beat = rs.transport.lpb
    local seconds_per_beat = 60 / beats_per_minute
    local sample_frames = rs.selected_sample.sample_buffer.number_of_frames
    local sample_rate = rs.selected_sample.sample_buffer.sample_rate
    local sample_seconds = sample_frames / sample_rate
    local sample_beats = sample_seconds / seconds_per_beat
    local sample_lines = sample_beats * lines_per_beat
    local pattern_lines = math.min(512, math.max(1, round(sample_lines)))
    if (pattern_lines ~= sample_lines and set_bpm == true) then
      beats_per_minute = beats_per_minute * (pattern_lines / sample_lines)
      rs.transport.bpm = math.min(999, math.max(32, beats_per_minute))
    end
    rs.selected_pattern.number_of_lines = pattern_lines
  end
end

local function show_dialog()

  if dialog and dialog.visible then
    dialog:show()
    return
  end

  vb = renoise.ViewBuilder()
  
  local DIALOG_MARGIN = renoise.ViewBuilder.DEFAULT_DIALOG_MARGIN
  local DIALOG_SPACING = renoise.ViewBuilder.DEFAULT_DIALOG_SPACING
  local CONTROL_MARGIN = renoise.ViewBuilder.DEFAULT_CONTROL_MARGIN
    
  local content = vb:column {
  
    width = '100%',
    
    uniform = true,
    style = 'body',
    margin = DIALOG_MARGIN,
    spacing = DIALOG_SPACING,  
            
    vb:button {
      width = '100%',
      height = 32,
      text = 'Set Pattern Length',
      notifier = function()
        set_pattern_length_from_sample(false)
      end
    },
    
    vb:button {
      width = '100%',
      height = 32,
      text = 'Set Pattern Length and BPM',
      notifier = function()
        set_pattern_length_from_sample(true)
      end
    },
    
    vb:button {
      width = '100%',
      height = 32,
      text = 'Set BPM',
      notifier = function()
        set_bpm_from_sample()
      end
    },

  } 
  
  dialog = renoise.app():show_custom_dialog(tool_name .. ' v' .. tool_version, content)  
end

renoise.tool():add_menu_entry {
  name = "Pattern Editor:Pattern:Set Pattern Length From Sample",
  invoke = function()
    show_dialog()
  end
}

renoise.tool():add_keybinding {
  name = "Pattern Editor:Pattern:Set Pattern Length From Sample",
  invoke = function()
    show_dialog()
  end
}
